/* -*- mode: c++; c-basic-offset: 4; -*- */
#include "file_driver.hh"
#include "loc_list.hh"
#include "helpers.hh"
#include <stdexcept>

#include "framecpp/IFrameStream.hh"

#define FCPP_SHARED_PTRS 1
#include "ldastoolsal/SharedPtr.hh"
using LDASTools::AL::SharedPtr;

#include <fstream>
#include <iostream>
#include <unistd.h>

using namespace std;
using namespace sends;

//======================================  Constructor
file_driver::file_driver(void) {
}

//======================================  Destructor
file_driver::~file_driver(void) {
}

//======================================  Read data into segments
int
file_driver::read_data(const loc_list& ll, gps_type start, 
		       gps_type stop, const channel_vect& v) {
    int rc = 0;

    frame_loc fl;
    if (!ll.find_loc(start, fl)) {
	cerr << "GPS not in loc_list" << endl;
	return -1;
    }
    if (stop > fl.stop()) stop = fl.stop();
    string prefix = fl.prefix();

    //----------------------------------  Loop over frames
    segment_vect segv;
    while (start < stop) {
	if (!ll.find_loc(start, fl)) {
	    cerr << "GPS not in loc_list" << endl;
	    rc = -1;
	    break;
	}

	string file = fl.file_path(start);
	gps_type end_frame   = fl.frame_end(start);
	gps_type start_frame = fl.frame_start(start);

	if (file.empty()) {
	    cerr << "file_driver: Can't find frame for gps= " << start 
		 << " in " << loc_list::arch_name(ll.archive()) << ":" 
		 << prefix << endl;
	    rc = -1;
	    break;
	} else if (access(file.c_str(), R_OK) != 0) {
	    cerr << "file_driver: Can't find frame: " << file << endl;
	    rc = -1;
	    break;
	}

	if (!get_segments(start_frame, end_frame, v, segv)) {
	    start = end_frame;
	    continue;
	}

	try {
	    //--------------------------  Open the frame
	    if (debug() >= 2) cerr << "Read data from frame: " << file << endl;
	    SharedPtr< FrameCPP::Common::FrameBuffer<filebuf> > 
	    frbuf( new FrameCPP::Common::FrameBuffer<filebuf>(ios::in) );
	    if (! frbuf.get()->open(file.c_str(), ios::in) ) 
		throw runtime_error( string("Unable to open file: ") + file);

		// ensure that we always call the close method
		auto_closer< FrameCPP::Common::FrameBuffer<filebuf> > _closer(*frbuf);

	    FrameCPP::IFrameStream frdr(false, frbuf.get());

	    //--------------------------  Loop over frames, read data.
	    int nFrames = frdr.GetTOC()->nFrame();
	    for (int iFrame=0; iFrame < nFrames; ++iFrame) {
		if (read_frame(frdr, segv, iFrame, start_frame)) {
		    cerr << "Error reading frame: " << file << endl;
		    rc = -1;
		}
	    }

	    //--------------------------  Finish processing
	    start = end_frame;

	//------------------------------  On error, release segments and 
	//                                rethrow exception
	} catch (std::exception& e) {
	    release_segments(segv);
	    cerr << "file_driver::read_data: caught exception: " 
		 << e.what() << endl;
	    rc = -1;
	    break;
	} catch (...) {
	    release_segments(segv);
	    throw;
	}
	release_segments(segv);
    }
    return rc;
}
